/*  <lalVerbatim file="LALInspiralComputeChisqCV">
Author: Sathyaprakash, B. S.
$Id: LALInspiralComputeChisq.c,v 1.1 2002/11/27 01:58:28 sathya Exp $
</lalVerbatim>  */

/* <lalLaTeX>
\subsection{Module \texttt{LALInspiralComputeChisq.c}}

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{LALInspiralComputeChisqCP}
\index{\verb&LALInspiralComputeChisq()&}

\subsubsection*{Description}
\subsubsection*{Algorithm}
\subsubsection*{Uses}
\begin{verbatim}
LALREAL4VectorFFT
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{LALInspiralComputeChisqCV}}
</lalLaTeX>  */

#include <lal/LALNoiseModels.h>

NRCSID (LALINSPIRALCOMPUTECHISQ, "$Id: LALInspiralComputeChisq.c,v 1.1 2002/11/27 01:58:28 sathya Exp $");

/*  <lalVerbatim file="LALInspiralComputeChisqCP"> */
void
LALInspiralComputeChisq
   (
   LALStatus     *status,
   REAL4         *chisq,
   InspiralChisqDataVec  *input,
   InspiralChisqParams   *params
   )
{  /*  </lalVerbatim>  */

  REAL8 flso, df, duration, totalModelSNR, SNRPerBin, binSNR, totalSNR, diffSNR, mSevenBy3;
  INT4 n, k_0, k_N, ki, count, k, k_start, k_end, i;
  UINT4 binIndexLength;
  INT4Vector *binIndexes = NULL;

  INITSTATUS (status, "LALInspiralComputeChisq", LALINSPIRALCOMPUTECHISQ);
  ATTATCHSTATUSPTR(status);

/* Need to calculate the total SNR from the stationary phase approximation */



  binIndexLength = params->nBins + 1;
  LALI4CreateVector(status->statusPtr, &binIndexes, binIndexLength);
  
  flso = 1.0/(pow(6.0,3.0/2.0)*LAL_PI*params->totalMass);

  duration = 2.L * (input->psd->length-1)*params->deltaT;
  df = 1.L/duration;

  k_0 = floor(params->fLower/df);  /* The lower discrete frequency in summation */
  k_N = floor(flso/df);            /* The upper discrete frequency in summation */

  n = input->SNRIntegrand->length;

  /*
  fprintf(stderr, "%d %d %d\n", n, k_0, k_N);
   */
  
  totalModelSNR = 0.0;
  mSevenBy3 = -7.L/3.L;
  for(k=k_0; k<=k_N; k++)
  {
     if (input->psd->data[k]) totalModelSNR += pow( (float)k*df, mSevenBy3) / input->psd->data[k];
  }

  SNRPerBin = totalModelSNR/(double)params->nBins;

  count = 0;
  binIndexes->data[count] = k_0;
  k_start = k_0;

  /* At the end of this loop we should have the INT4Vector binIndexes filled up
   * with the indexes k corresponding to the boundaries between the chi-squared bins
   */

  k = k_start;
  while(k < k_N)
  {
     binSNR = 0.0;
     for(k=k_start; k<=k_N; k++)
     {
        if (input->psd->data[k]) binSNR += pow( (float)k*df, mSevenBy3) / input->psd->data[k];

           if (binSNR > SNRPerBin)
	   {  
		   count++;
		   binIndexes->data[count] = k;
		   k_start = k+1;
		   break;
	   }
     }
  }
  count++;
  binIndexes->data[count] = k_N;

  if (count != params->nBins) 
  {
	  fprintf(stderr, "count=%d not same as binIndexLength=%d\n", count, binIndexLength);
  }


  /*
  for(k=1; k < input->psd->length; k++)
  {
     if (input->psd->data[k]) 
	     fprintf(stdout, "%e %e \n", (float)k*df, (pow((float)k * df, mSevenBy3) / input->psd->data[k]));
  }
     fprintf(stdout, "&\n");
   for(i=0; i < binIndexLength; i++)
   {
      fprintf(stdout, "%e %d\n", binIndexes->data[i]*df, i);
   } 
  
   exit(0);
   */

  /* Now get the total SNR from the data */

  totalSNR = 0.0;
  for(k=k_0; k<=k_N; k++)
  {
     totalSNR += input->SNRIntegrand->data[k] + input->SNRIntegrand->data[n-k];
  }
  /* Now count the negative frequencies too */
  totalSNR *= 2.;

  /* the expected SNR per chisq bin in the frequency space */

  SNRPerBin = totalSNR / (double)params->nBins;

  /* Now we need to sum up the SNR in each of the bins defined and stored in binIndexes->data[k] 
   * Then we take the difference between this figure and the SNRPerBin calculated immediately
   * above, and use this to compute the chi-squared statistic.
   */

   *chisq = 0.0;
   for(i=0; i<params->nBins; i++)
   {
      k_start = binIndexes->data[i];
      k_end = binIndexes->data[i+1];

      binSNR = 0.0;
      for(k=k_start; k<=k_end; k++)
      {
         binSNR += input->SNRIntegrand->data[k] + input->SNRIntegrand->data[n-k];
      }

      binSNR *= 2.;
      diffSNR = 1. - binSNR/SNRPerBin;
      *chisq += diffSNR * diffSNR;
   }
   /*
   fprintf(stderr, "snr=%e chisq=%e\n", totalSNR, *chisq);
   */
      
  LALI4DestroyVector(status->statusPtr, &binIndexes);
  DETATCHSTATUSPTR(status);
  RETURN(status);
}
